@@ -25,28 +25,27 @@
  */
 
 /**
- * @page cpuid_tool
- * @date 2008-11-19
- * @author Veselin Georgiev
- * @brief Command line interface to libcpuid
+ * @File cpuid_tool.c
+ * @Date 2008-11-19
+ * @Author Veselin Georgiev
+ * @Brief Command line interface to libcpuid
  *
- * @details
  * This file is provides a direct interface to libcpuid. See the usage()
  * function (or just run the program with the `--help' switch) for a short
  * command line options reference.
  *
  * This file has several purposes:
  *
- * 1. When started with no arguments, the program outputs the RAW and decoded
+ * 1) When started with no arguments, the program outputs the RAW and decoded
  *    CPU data to files (`raw.txt' and `report.txt', respectively) - this is
  *    intended to be a dumb, doubleclicky tool for non-developer
  *    users, that can provide debug info about unrecognized processors to
  *    libcpuid developers.
- * 2. When operated from the terminal with the `--report' option, it is a
+ * 2) When operated from the terminal with the `--report' option, it is a
  *    generic CPU-info utility.
- * 3. Can be used in shell scripts, e.g. to get the name of the CPU, cache
+ * 3) Can be used in shell scripts, e.g. to get the name of the CPU, cache
  *    sizes, features, with query options like `--cache', `--brandstr', etc.
- * 4. Finally, it serves to self-document libcpiud itself :)
+ * 4) Finally, it serves to self-document libcpiud itself :)
  */
 #include <stdio.h>
 #include <string.h>
@@ -73,15 +72,12 @@ typedef enum {
 	NEED_L1I_SIZE,
 	NEED_L2_SIZE,
 	NEED_L3_SIZE,
-	NEED_L4_SIZE,
 	NEED_L1D_ASSOC,
 	NEED_L2_ASSOC,
 	NEED_L3_ASSOC,
-	NEED_L4_ASSOC,
 	NEED_L1D_CACHELINE,
 	NEED_L2_CACHELINE,
 	NEED_L3_CACHELINE,
-	NEED_L4_CACHELINE,
 	NEED_CODENAME,
 	NEED_FEATURES,
 	NEED_CLOCK,
@@ -89,7 +85,6 @@ typedef enum {
 	NEED_CLOCK_RDTSC,
 	NEED_CLOCK_IC,
 	NEED_RDMSR,
-	NEED_RDMSR_RAW,
 	NEED_SSE_UNIT_SIZE,
 } output_data_switch;
 
@@ -101,9 +96,7 @@ int need_input = 0,
     need_timed_clockreport = 0,
     verbose_level = 0,
     need_version = 0,
-    need_cpulist = 0,
-    need_sgx = 0,
-    need_identify = 0;
+    need_cpulist = 0;
 
 #define MAX_REQUESTS 32
 int num_requests = 0;
@@ -131,15 +124,12 @@ matchtable[] = {
 	{ NEED_L2_SIZE      , "--cache"        , 1},
 	{ NEED_L2_SIZE      , "--l2-cache"     , 1},
 	{ NEED_L3_SIZE      , "--l3-cache"     , 1},
-	{ NEED_L4_SIZE      , "--l4-cache"     , 1},
 	{ NEED_L1D_ASSOC    , "--l1d-assoc"    , 1},
 	{ NEED_L2_ASSOC     , "--l2-assoc"     , 1},
 	{ NEED_L3_ASSOC     , "--l3-assoc"     , 1},
-	{ NEED_L4_ASSOC     , "--l4-assoc"     , 1},
 	{ NEED_L1D_CACHELINE, "--l1d-cacheline", 1},
 	{ NEED_L2_CACHELINE , "--l2-cacheline" , 1},
 	{ NEED_L3_CACHELINE , "--l3-cacheline" , 1},
-	{ NEED_L4_CACHELINE , "--l4-cacheline" , 1},
 	{ NEED_CODENAME     , "--codename"     , 1},
 	{ NEED_FEATURES     , "--flags"        , 1},
 	{ NEED_CLOCK        , "--clock"        , 0},
@@ -147,7 +137,6 @@ matchtable[] = {
 	{ NEED_CLOCK_RDTSC  , "--clock-rdtsc"  , 1},
 	{ NEED_CLOCK_IC     , "--clock-ic"     , 1},
 	{ NEED_RDMSR        , "--rdmsr"        , 0},
-	{ NEED_RDMSR_RAW    , "--rdmsr-raw"    , 0},
 	{ NEED_SSE_UNIT_SIZE, "--sse-size"     , 1},
 };
 
@@ -167,7 +156,6 @@ static void usage(void)
 	printf("  --clock          - in conjunction to --report: print CPU clock as well\n");
 	printf("  --clock-rdtsc    - same as --clock, but use RDTSC for clock detection\n");
 	printf("  --cpulist        - list all known CPUs\n");
-	printf("  --sgx            - list SGX leaf data, if SGX is supported.\n");
 	printf("  --quiet          - disable warnings\n");
 	printf("  --outfile=<file> - redirect all output to this file, instead of stdout\n");
 	printf("  --verbose, -v    - be extra verbose (more keys increase verbosiness level)\n");
@@ -282,11 +270,6 @@ static int parse_cmdline(int argc, char** argv)
 			need_cpulist = 1;
 			recog = 1;
 		}
-		if (!strcmp(arg, "--sgx")) {
-			need_sgx = 1;
-			need_identify = 1;
-			recog = 1;
-		}
 		if (arg[0] == '-' && arg[1] == 'v') {
 			num_vs = 1;
 			while (arg[num_vs] == 'v')
@@ -324,7 +307,7 @@ static int check_need_raw_data(void)
 {
 	int i, j;
 	
-	if (need_output || need_report || need_identify) return 1;
+	if (need_output || need_report) return 1;
 	for (i = 0; i < num_requests; i++) {
 		for (j = 0; j < sz_match; j++)
 			if (requests[i] == matchtable[j].sw &&
@@ -336,7 +319,7 @@ static int check_need_raw_data(void)
 static void print_info(output_data_switch query, struct cpu_raw_data_t* raw,
                        struct cpu_id_t* data)
 {
-	int i, value;
+	int i;
 	struct msr_driver_t* handle;
 	switch (query) {
 		case NEED_CPUID_PRESENT:
@@ -373,7 +356,7 @@ static void print_info(output_data_switch query, struct cpu_raw_data_t* raw,
 			fprintf(fout, "%d\n", data->num_logical_cpus);
 			break;
 		case NEED_TOTAL_CPUS:
-			fprintf(fout, "%d\n", cpuid_get_total_cpus());
+			fprintf(fout, "%d\n", data->total_logical_cpus);
 			break;
 		case NEED_L1D_SIZE:
 			fprintf(fout, "%d\n", data->l1_data_cache);
@@ -387,9 +370,6 @@ static void print_info(output_data_switch query, struct cpu_raw_data_t* raw,
 		case NEED_L3_SIZE:
 			fprintf(fout, "%d\n", data->l3_cache);
 			break;
-		case NEED_L4_SIZE:
-			fprintf(fout, "%d\n", data->l4_cache);
-			break;
 		case NEED_L1D_ASSOC:
 			fprintf(fout, "%d\n", data->l1_assoc);
 			break;
@@ -399,9 +379,6 @@ static void print_info(output_data_switch query, struct cpu_raw_data_t* raw,
 		case NEED_L3_ASSOC:
 			fprintf(fout, "%d\n", data->l3_assoc);
 			break;
-		case NEED_L4_ASSOC:
-			fprintf(fout, "%d\n", data->l4_assoc);
-			break;
 		case NEED_L1D_CACHELINE:
 			fprintf(fout, "%d\n", data->l1_cacheline);
 			break;
@@ -411,9 +388,6 @@ static void print_info(output_data_switch query, struct cpu_raw_data_t* raw,
 		case NEED_L3_CACHELINE:
 			fprintf(fout, "%d\n", data->l3_cacheline);
 			break;
-		case NEED_L4_CACHELINE:
-			fprintf(fout, "%d\n", data->l4_cacheline);
-			break;
 		case NEED_CODENAME:
 			fprintf(fout, "%s\n", data->cpu_codename);
 			break;
@@ -442,34 +416,10 @@ static void print_info(output_data_switch query, struct cpu_raw_data_t* raw,
 			if ((handle = cpu_msr_driver_open()) == NULL) {
 				fprintf(fout, "Cannot open MSR driver: %s\n", cpuid_error());
 			} else {
-				if ((value = cpu_msrinfo(handle, INFO_MPERF)) != CPU_INVALID_VALUE)
-					fprintf(fout, "  MSR.mperf  : %d MHz\n", value);
-				if ((value = cpu_msrinfo(handle, INFO_APERF)) != CPU_INVALID_VALUE)
-					fprintf(fout, "  MSR.aperf  : %d MHz\n", value);
-				if ((value = cpu_msrinfo(handle, INFO_MIN_MULTIPLIER)) != CPU_INVALID_VALUE)
-					fprintf(fout, "  min. multi.: %.2lf\n", value / 100.0);
-				if ((value = cpu_msrinfo(handle, INFO_CUR_MULTIPLIER)) != CPU_INVALID_VALUE)
-					fprintf(fout, "  cur. multi.: %.2lf\n", value / 100.0);
-				if ((value = cpu_msrinfo(handle, INFO_MAX_MULTIPLIER)) != CPU_INVALID_VALUE)
-					fprintf(fout, "  max. multi.: %.2lf\n", value / 100.0);
-				if ((value = cpu_msrinfo(handle, INFO_TEMPERATURE)) != CPU_INVALID_VALUE)
-					fprintf(fout, "  temperature: %d degrees Celsius\n", value);
-				if ((value = cpu_msrinfo(handle, INFO_THROTTLING)) != CPU_INVALID_VALUE)
-					fprintf(fout, "  throttling : %s\n", value ? "yes" : "no");
-				if ((value = cpu_msrinfo(handle, INFO_VOLTAGE)) != CPU_INVALID_VALUE)
-					fprintf(fout, "  core volt. : %.2lf Volts\n", value / 100.0);
-				if ((value = cpu_msrinfo(handle, INFO_BUS_CLOCK)) != CPU_INVALID_VALUE)
-					fprintf(fout, "  bus clock  : %.2lf MHz\n", value / 100.0);
-				cpu_msr_driver_close(handle);
-			}
-			break;
-		}
-		case NEED_RDMSR_RAW:
-		{
-			if ((handle = cpu_msr_driver_open()) == NULL) {
-				fprintf(fout, "Cannot open MSR driver: %s\n", cpuid_error());
-			} else {
-				msr_serialize_raw_data(handle, "");
+				fprintf(fout, "mperf = %d\n", cpu_msrinfo(handle, INFO_MPERF));
+				fprintf(fout, "aperf = %d\n", cpu_msrinfo(handle, INFO_APERF));
+				fprintf(fout, "current cpu:fsb = %.1f\n", cpu_msrinfo(handle, INFO_CUR_MULTIPLIER) / 100.0f);
+				fprintf(fout, "max cpu:fsb = %.1f\n", cpu_msrinfo(handle, INFO_MAX_MULTIPLIER) / 100.0f);
 				cpu_msr_driver_close(handle);
 			}
 			break;
@@ -511,27 +461,6 @@ static void print_cpulist(void)
 	}
 }
 
-static void print_sgx_data(const struct cpu_raw_data_t* raw, const struct cpu_id_t* data)
-{
-	int i;
-	fprintf(fout, "SGX: %d (%s)\n", data->sgx.present, data->sgx.present ? "present" : "absent");
-	if (data->sgx.present) {
-		fprintf(fout, "SGX max enclave size (32-bit): 2^%d\n", data->sgx.max_enclave_32bit);
-		fprintf(fout, "SGX max enclave size (64-bit): 2^%d\n", data->sgx.max_enclave_64bit);
-		fprintf(fout, "SGX1 extensions              : %d (%s)\n", data->sgx.flags[INTEL_SGX1], data->sgx.flags[INTEL_SGX1] ? "present" : "absent");
-		fprintf(fout, "SGX2 extensions              : %d (%s)\n", data->sgx.flags[INTEL_SGX2], data->sgx.flags[INTEL_SGX2] ? "present" : "absent");
-		fprintf(fout, "SGX MISCSELECT               : %08x\n", data->sgx.misc_select);
-		fprintf(fout, "SGX SECS.ATTRIBUTES mask     : %016llx\n", (unsigned long long) data->sgx.secs_attributes);
-		fprintf(fout, "SGX SECS.XSAVE feature mask  : %016llx\n", (unsigned long long) data->sgx.secs_xfrm);
-		fprintf(fout, "SGX EPC sections count       : %d\n", data->sgx.num_epc_sections);
-		for (i = 0; i < data->sgx.num_epc_sections; i++) {
-			struct cpu_epc_t epc = cpuid_get_epc(i, raw);
-			fprintf(fout, "    SGX EPC section #%-8d: start = %llx, size = %llu\n", i,
-				(unsigned long long) epc.start_addr, (unsigned long long) epc.length);
-		}
-	}
-}
-
 int main(int argc, char** argv)
 {
 	int parseres = parse_cmdline(argc, argv);
@@ -653,15 +582,12 @@ int main(int argc, char** argv)
 		fprintf(fout, "  L1 I cache : %d KB\n", data.l1_instruction_cache);
 		fprintf(fout, "  L2 cache   : %d KB\n", data.l2_cache);
 		fprintf(fout, "  L3 cache   : %d KB\n", data.l3_cache);
-		fprintf(fout, "  L4 cache   : %d KB\n", data.l4_cache);
 		fprintf(fout, "  L1D assoc. : %d-way\n", data.l1_assoc);
 		fprintf(fout, "  L2 assoc.  : %d-way\n", data.l2_assoc);
 		fprintf(fout, "  L3 assoc.  : %d-way\n", data.l3_assoc);
-		fprintf(fout, "  L4 assoc.  : %d-way\n", data.l4_assoc);
 		fprintf(fout, "  L1D line sz: %d bytes\n", data.l1_cacheline);
 		fprintf(fout, "  L2 line sz : %d bytes\n", data.l2_cacheline);
 		fprintf(fout, "  L3 line sz : %d bytes\n", data.l3_cacheline);
-		fprintf(fout, "  L4 line sz : %d bytes\n", data.l4_cacheline);
 		fprintf(fout, "  SSE units  : %d bits (%s)\n", data.sse_size, data.detection_hints[CPU_HINT_SSE_SIZE_AUTH] ? "authoritative" : "non-authoritative");
 		fprintf(fout, "  code name  : `%s'\n", data.cpu_codename);
 		fprintf(fout, "  features   :");
@@ -701,7 +627,7 @@ int main(int argc, char** argv)
 			break;
 		}
 	/* OK, process all queries. */
-	if (((!need_report || !only_clock_queries) && num_requests > 0) || need_identify) {
+	if ((!need_report || !only_clock_queries) && num_requests > 0) {
 		/* Identify the CPU. Make it do cpuid_get_raw_data() itself */
 		if (check_need_raw_data() && cpu_identify(&raw, &data) < 0) {
 			if (!need_quiet)
@@ -716,9 +642,6 @@ int main(int argc, char** argv)
 	if (need_cpulist) {
 		print_cpulist();
 	}
-	if (need_sgx) {
-		print_sgx_data(&raw, &data);
-	}
 	
 	return 0;
 }
